/*=========================================================================
| Beagle Interface Library
|--------------------------------------------------------------------------
| Copyright (c) 2004-2008 Total Phase, Inc.
| All rights reserved.
| www.totalphase.com
|
| Redistribution and use in source and binary forms, with or without
| modification, are permitted provided that the following conditions
| are met:
|
| - Redistributions of source code must retain the above copyright
|   notice, this list of conditions and the following disclaimer.
|
| - Redistributions in binary form must reproduce the above copyright
|   notice, this list of conditions and the following disclaimer in the
|   documentation and/or other materials provided with the distribution.
|
| - Neither the name of Total Phase, Inc. nor the names of its
|   contributors may be used to endorse or promote products derived from
|   this software without specific prior written permission.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
|--------------------------------------------------------------------------
| To access Total Phase Beagle devices through the API:
|
| 1) Use one of the following shared objects:
|      beagle.so        --  Linux shared object
|          or
|      beagle.dll       --  Windows dynamic link library
|
| 2) Along with one of the following language modules:
|      beagle.c/h       --  C/C++ API header file and interface module
|      beagle_py.py     --  Python API
|      beagle.bas       --  Visual Basic 6 API
|      beagle.cs        --  C# .NET source
|      beagle_net.dll   --  Compiled .NET binding
 ========================================================================*/

using System;
using System.Reflection;
using System.Runtime.InteropServices;

[assembly: AssemblyTitleAttribute("Beagle .NET binding")]
[assembly: AssemblyDescriptionAttribute(".NET binding for Beagle")]
[assembly: AssemblyCompanyAttribute("Total Phase, Inc.")]
[assembly: AssemblyProductAttribute("Beagle")]
[assembly: AssemblyCopyrightAttribute("Total Phase, Inc. 2010")]

namespace TotalPhase {

public enum BeagleStatus : int {
    /* General codes (0 to -99) */
    BG_OK                      =    0,
    BG_UNABLE_TO_LOAD_LIBRARY  =   -1,
    BG_UNABLE_TO_LOAD_DRIVER   =   -2,
    BG_UNABLE_TO_LOAD_FUNCTION =   -3,
    BG_INCOMPATIBLE_LIBRARY    =   -4,
    BG_INCOMPATIBLE_DEVICE     =   -5,
    BG_INCOMPATIBLE_DRIVER     =   -6,
    BG_COMMUNICATION_ERROR     =   -7,
    BG_UNABLE_TO_OPEN          =   -8,
    BG_UNABLE_TO_CLOSE         =   -9,
    BG_INVALID_HANDLE          =  -10,
    BG_CONFIG_ERROR            =  -11,
    BG_UNKNOWN_PROTOCOL        =  -12,
    BG_STILL_ACTIVE            =  -13,
    BG_FUNCTION_NOT_AVAILABLE  =  -14,

    /* COMMTEST codes (-100 to -199) */
    BG_COMMTEST_NOT_AVAILABLE  = -100,
    BG_COMMTEST_NOT_ENABLED    = -101,

    /* I2C codes (-200 to -299) */
    BG_I2C_NOT_AVAILABLE       = -200,
    BG_I2C_NOT_ENABLED         = -201,

    /* SPI codes (-300 to -399) */
    BG_SPI_NOT_AVAILABLE       = -300,
    BG_SPI_NOT_ENABLED         = -301,

    /* USB codes (-400 to -499) */
    BG_USB_NOT_AVAILABLE       = -400,
    BG_USB_NOT_ENABLED         = -401,

    /* MDIO codes (-500 to -599) */
    BG_MDIO_NOT_AVAILABLE      = -500,
    BG_MDIO_NOT_ENABLED        = -501,
    BG_MDIO_BAD_TURNAROUND     = -502
}

public enum BeagleProtocol : int {
    BG_PROTOCOL_NONE     = 0,
    BG_PROTOCOL_COMMTEST = 1,
    BG_PROTOCOL_USB      = 2,
    BG_PROTOCOL_I2C      = 3,
    BG_PROTOCOL_SPI      = 4,
    BG_PROTOCOL_MDIO     = 5
}

public enum BeagleSpiSSPolarity : int {
    BG_SPI_SS_ACTIVE_LOW  = 0,
    BG_SPI_SS_ACTIVE_HIGH = 1
}

public enum BeagleSpiSckSamplingEdge : int {
    BG_SPI_SCK_SAMPLING_EDGE_RISING  = 0,
    BG_SPI_SCK_SAMPLING_EDGE_FALLING = 1
}

public enum BeagleSpiBitorder : int {
    BG_SPI_BITORDER_MSB = 0,
    BG_SPI_BITORDER_LSB = 1
}

public enum BeagleUsb480CaptureMode : int {
    BG_USB480_CAPTURE_REALTIME                 = 0,
    BG_USB480_CAPTURE_REALTIME_WITH_PROTECTION = 1,
    BG_USB480_CAPTURE_DELAYED_DOWNLOAD         = 2
}

public enum BeagleUsb480TargetSpeed : int {
    BG_USB480_AUTO_SPEED_DETECT = 0,
    BG_USB480_LOW_SPEED         = 1,
    BG_USB480_FULL_SPEED        = 2,
    BG_USB480_HIGH_SPEED        = 3
}

public enum BeagleUsb480DigitalOutMatchPins : int {
    BG_USB480_DIGITAL_OUT_MATCH_PIN3 = 3,
    BG_USB480_DIGITAL_OUT_MATCH_PIN4 = 4
}

public enum BeagleUsb480MatchType : int {
    BG_USB480_MATCH_TYPE_DISABLED  = 0,
    BG_USB480_MATCH_TYPE_EQUAL     = 1,
    BG_USB480_MATCH_TYPE_NOT_EQUAL = 2
}

public enum BeagleMdioClause : int {
    BG_MDIO_CLAUSE_22    = 0,
    BG_MDIO_CLAUSE_45    = 1,
    BG_MDIO_CLAUSE_ERROR = 2
}


public class BeagleApi {

/*=========================================================================
| HELPER FUNCTIONS / CLASSES
 ========================================================================*/
static long tp_min(long x, long y) { return x < y ? x : y; }

private class GCContext {
    GCHandle[] handles;
    int index;
    public GCContext () {
        handles = new GCHandle[16];
        index   = 0;
    }
    public void add (GCHandle gch) {
        handles[index] = gch;
        index++;
    }
    public void free () {
        while (index != 0) {
            index--;
            handles[index].Free();
        }
    }
}

/*=========================================================================
| VERSION
 ========================================================================*/
[DllImport ("beagle")]
private static extern int c_version ();

public const int BG_API_VERSION    = 0x0304;   // v3.04
public const int BG_REQ_SW_VERSION = 0x0307;   // v3.07

private static short BG_SW_VERSION;
private static short BG_REQ_API_VERSION;
private static bool  BG_LIBRARY_LOADED;

static BeagleApi () {
    BG_SW_VERSION      = (short)(c_version() & 0xffff);
    BG_REQ_API_VERSION = (short)((c_version() >> 16) & 0xffff);
    BG_LIBRARY_LOADED  = 
        ((BG_SW_VERSION >= BG_REQ_SW_VERSION) &&
         (BG_API_VERSION >= BG_REQ_API_VERSION));
}

/*=========================================================================
| STATUS CODES
 ========================================================================*/
/*
 * All API functions return an integer which is the result of the
 * transaction, or a status code if negative.  The status codes are
 * defined as follows:
 */
// enum BeagleStatus  (from declaration above)
//     BG_OK                      =    0
//     BG_UNABLE_TO_LOAD_LIBRARY  =   -1
//     BG_UNABLE_TO_LOAD_DRIVER   =   -2
//     BG_UNABLE_TO_LOAD_FUNCTION =   -3
//     BG_INCOMPATIBLE_LIBRARY    =   -4
//     BG_INCOMPATIBLE_DEVICE     =   -5
//     BG_INCOMPATIBLE_DRIVER     =   -6
//     BG_COMMUNICATION_ERROR     =   -7
//     BG_UNABLE_TO_OPEN          =   -8
//     BG_UNABLE_TO_CLOSE         =   -9
//     BG_INVALID_HANDLE          =  -10
//     BG_CONFIG_ERROR            =  -11
//     BG_UNKNOWN_PROTOCOL        =  -12
//     BG_STILL_ACTIVE            =  -13
//     BG_FUNCTION_NOT_AVAILABLE  =  -14
//     BG_COMMTEST_NOT_AVAILABLE  = -100
//     BG_COMMTEST_NOT_ENABLED    = -101
//     BG_I2C_NOT_AVAILABLE       = -200
//     BG_I2C_NOT_ENABLED         = -201
//     BG_SPI_NOT_AVAILABLE       = -300
//     BG_SPI_NOT_ENABLED         = -301
//     BG_USB_NOT_AVAILABLE       = -400
//     BG_USB_NOT_ENABLED         = -401
//     BG_MDIO_NOT_AVAILABLE      = -500
//     BG_MDIO_NOT_ENABLED        = -501
//     BG_MDIO_BAD_TURNAROUND     = -502


/*=========================================================================
| GENERAL TYPE DEFINITIONS
 ========================================================================*/
/* Beagle handle type definition */
/* typedef Beagle => int */

/*
 * Beagle version matrix.
 *
 * This matrix describes the various version dependencies
 * of Beagle components.  It can be used to determine
 * which component caused an incompatibility error.
 *
 * All version numbers are of the format:
 *   (major << 8) | minor
 *
 * ex. v1.20 would be encoded as:  0x0114
 */
[StructLayout(LayoutKind.Sequential)]
public struct BeagleVersion {
    /* Software, firmware, and hardware versions. */
    public ushort software;
    public ushort firmware;
    public ushort hardware;

    /*
     * Hardware revisions that are compatible with this software version.
     * The top 16 bits gives the maximum accepted hw revision.
     * The lower 16 bits gives the minimum accepted hw revision.
     */
    public uint   hw_revs_for_sw;

    /*
     * Firmware revisions that are compatible with this software version.
     * The top 16 bits gives the maximum accepted fw revision.
     * The lower 16 bits gives the minimum accepted fw revision.
     */
    public uint   fw_revs_for_sw;

    /*
     * Driver revisions that are compatible with this software version.
     * The top 16 bits gives the maximum accepted driver revision.
     * The lower 16 bits gives the minimum accepted driver revision.
     * This version checking is currently only pertinent for WIN32
     * platforms.
     */
    public uint   drv_revs_for_sw;

    /* Software requires that the API interface must be >= this version. */
    public ushort api_req_by_sw;
}


/*=========================================================================
| GENERAL API
 ========================================================================*/
/*
 * Get a list of ports to which Beagle devices are attached.
 *
 * num_devices = maximum number of elements to return
 * devices     = array into which the port numbers are returned
 *
 * Each element of the array is written with the port number.
 * Devices that are in-use are ORed with BG_PORT_NOT_FREE
 * (0x8000).
 *
 * ex.  devices are attached to ports 0, 1, 2
 *      ports 0 and 2 are available, and port 1 is in-use.
 *      array => 0x0000, 0x8001, 0x0002
 *
 * If the array is NULL, it is not filled with any values.
 * If there are more devices than the array size, only the
 * first nmemb port numbers will be written into the array.
 *
 * Returns the number of devices found, regardless of the
 * array size.
 */
public const ushort BG_PORT_NOT_FREE = 0x8000;
public static int bg_find_devices (
    int       num_devices,
    ushort[]  devices
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int devices_num_devices = (int)tp_min(num_devices, devices.Length);
    return net_bg_find_devices(devices_num_devices, devices);
}

/*
 * Get a list of ports to which Beagle devices are attached
 *
 * This function is the same as bg_find_devices() except that
 * it returns the unique IDs of each Beagle device.  The IDs
 * are guaranteed to be non-zero if valid.
 *
 * The IDs are the unsigned integer representation of the 10-digit
 * serial numbers.
 */
public static int bg_find_devices_ext (
    int       num_devices,
    ushort[]  devices,
    int       num_ids,
    uint[]    unique_ids
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int devices_num_devices = (int)tp_min(num_devices, devices.Length);
    int unique_ids_num_ids = (int)tp_min(num_ids, unique_ids.Length);
    return net_bg_find_devices_ext(devices_num_devices, devices, unique_ids_num_ids, unique_ids);
}

/*
 * Open the Beagle port.
 *
 * The port number is a zero-indexed integer.
 *
 * The port number is the same as that obtained from the
 * bg_find_devices() function above.
 *
 * Returns an Beagle handle, which is guaranteed to be
 * greater than zero if it is valid.
 *
 * This function is recommended for use in simple applications
 * where extended information is not required.  For more complex
 * applications, the use of bg_open_ext() is recommended.
 */
public static int bg_open (
    int  port_number
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_open(port_number);
}

/*
 * Open the Beagle port, returning extended information
 * in the supplied structure.  Behavior is otherwise identical
 * to bg_open() above.  If 0 is passed as the pointer to the
 * structure, this function is exactly equivalent to bg_open().
 *
 * The structure is zeroed before the open is attempted.
 * It is filled with whatever information is available.
 *
 * For example, if the hardware version is not filled, then
 * the device could not be queried for its version number.
 *
 * This function is recommended for use in complex applications
 * where extended information is required.  For more simple
 * applications, the use of bg_open() is recommended.
 */
[StructLayout(LayoutKind.Sequential)]
public struct BeagleExt {
    /* Version matrix */
    public BeagleVersion version;

    /* Features of this device. */
    public int           features;
}

public static int bg_open_ext (
    int            port_number,
    ref BeagleExt  bg_ext
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_open_ext(port_number, ref bg_ext);
}

/* Close the Beagle port. */
public static int bg_close (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_close(beagle);
}

/*
 * Return the port for this Beagle handle.
 *
 * The port number is a zero-indexed integer.
 */
public static int bg_port (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_port(beagle);
}

/*
 * Return the device features as a bit-mask of values, or
 * an error code if the handle is not valid.
 */
public const int BG_FEATURE_NONE = 0x00000000;
public const int BG_FEATURE_I2C = 0x00000001;
public const int BG_FEATURE_SPI = 0x00000002;
public const int BG_FEATURE_USB = 0x00000004;
public const int BG_FEATURE_MDIO = 0x00000008;
public const int BG_FEATURE_USB_HS = 0x00000010;
public static int bg_features (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_features(beagle);
}

public static int bg_unique_id_to_features (
    uint  unique_id
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_unique_id_to_features(unique_id);
}

/*
 * Return the unique ID for this Beagle adapter.
 * IDs are guaranteed to be non-zero if valid.
 * The ID is the unsigned integer representation of the
 * 10-digit serial number.
 */
public static uint bg_unique_id (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return 0;
    return net_bg_unique_id(beagle);
}

/*
 * Return the status string for the given status code.
 * If the code is not valid or the library function cannot
 * be loaded, return a NULL string.
 */
public static string bg_status_string (
    int  status
)
{
    if (!BG_LIBRARY_LOADED) return null;
    return Marshal.PtrToStringAnsi(net_bg_status_string(status));
}

/*
 * Return the version matrix for the device attached to the
 * given handle.  If the handle is 0 or invalid, only the
 * software and required api versions are set.
 */
public static int bg_version (
    int                beagle,
    ref BeagleVersion  version
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_version(beagle, ref version);
}

/*
 * Set the capture latency to the specified number of milliseconds.
 * This number determines the minimum time that a read call will
 * block if there is no available data.  Lower times result in
 * faster turnaround at the expense of reduced buffering.  Setting
 * this parameter too low can cause packets to be dropped.
 */
public static int bg_latency (
    int   beagle,
    uint  milliseconds
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_latency(beagle, milliseconds);
}

/*
 * Set the capture timeout to the specified number of milliseconds.
 * If any read call has a longer idle than this value, that call
 * will return with a count of 0 bytes.
 */
public static int bg_timeout (
    int   beagle,
    uint  milliseconds
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_timeout(beagle, milliseconds);
}

/*
 * Sleep for the specified number of milliseconds
 * Accuracy depends on the operating system scheduler
 * Returns the number of milliseconds slept
 */
public static uint bg_sleep_ms (
    uint  milliseconds
)
{
    if (!BG_LIBRARY_LOADED) return 0;
    return net_bg_sleep_ms(milliseconds);
}

/* Configure the target power pin. */
public const byte BG_TARGET_POWER_OFF = 0x00;
public const byte BG_TARGET_POWER_ON = 0x01;
public const byte BG_TARGET_POWER_QUERY = 0x80;
public static int bg_target_power (
    int   beagle,
    byte  power_flag
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_target_power(beagle, power_flag);
}

public const byte BG_HOST_IFCE_FULL_SPEED = 0x00;
public const byte BG_HOST_IFCE_HIGH_SPEED = 0x01;
public static int bg_host_ifce_speed (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_host_ifce_speed(beagle);
}

/* Returns the device address that the beagle is attached to. */
public static int bg_dev_addr (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_dev_addr(beagle);
}


/*=========================================================================
| BUFFERING API
 ==========================================================================
| Set the amount of buffering that is to be allocated on the PC.
| Pass zero to num_bytes to query the existing buffer siz*/
public static int bg_host_buffer_size (
    int   beagle,
    uint  num_bytes
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_host_buffer_size(beagle, num_bytes);
}

/* Query the amount of buffering that is unused and free for buffering. */
public static int bg_host_buffer_free (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_host_buffer_free(beagle);
}

/* Query the amount of buffering that is used and no longer available. */
public static int bg_host_buffer_used (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_host_buffer_used(beagle);
}

/* Benchmark the speed of the host to Beagle interface */
public static int bg_commtest (
    int  beagle,
    int  num_samples,
    int  delay_count
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_commtest(beagle, num_samples, delay_count);
}


/*=========================================================================
| MONITORING API
 ========================================================================*/
/* Protocol codes */
// enum BeagleProtocol  (from declaration above)
//     BG_PROTOCOL_NONE     = 0
//     BG_PROTOCOL_COMMTEST = 1
//     BG_PROTOCOL_USB      = 2
//     BG_PROTOCOL_I2C      = 3
//     BG_PROTOCOL_SPI      = 4
//     BG_PROTOCOL_MDIO     = 5

/* Common Beagle read status codes */
public const uint BG_READ_OK = 0x00000000;
public const uint BG_READ_TIMEOUT = 0x00000100;
public const uint BG_READ_ERR_MIDDLE_OF_PACKET = 0x00000200;
public const uint BG_READ_ERR_SHORT_BUFFER = 0x00000400;
public const uint BG_READ_ERR_PARTIAL_LAST_BYTE = 0x00000800;
public const uint BG_READ_ERR_PARTIAL_LAST_BYTE_MASK = 0x0000000f;
public const uint BG_READ_ERR_UNEXPECTED = 0x00001000;
/* Enable the Beagle monitor */
public static int bg_enable (
    int             beagle,
    BeagleProtocol  protocol
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_enable(beagle, protocol);
}

/* Disable the Beagle monitor */
public static int bg_disable (
    int  beagle
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_disable(beagle);
}

/* Set the sample rate in kilohertz. */
public static int bg_samplerate (
    int  beagle,
    int  samplerate_khz
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_samplerate(beagle, samplerate_khz);
}

/*
 * Get the number of bits for the given number of bytes in the
 * given protocol.
 * Use this to determine how large a bit_timing array to allocate
 * for bg_*_read_bit_timing functions.
 */
public static int bg_bit_timing_size (
    BeagleProtocol  protocol,
    int             num_data_bytes
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_bit_timing_size(protocol, num_data_bytes);
}


/*=========================================================================
| I2C API
 ========================================================================*/
/* Configure the I2C pullup resistors. */
public const byte BG_I2C_PULLUP_OFF = 0x00;
public const byte BG_I2C_PULLUP_ON = 0x01;
public const byte BG_I2C_PULLUP_QUERY = 0x80;
public static int bg_i2c_pullup (
    int   beagle,
    byte  pullup_flag
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_i2c_pullup(beagle, pullup_flag);
}

public const ushort BG_I2C_MONITOR_DATA = 0x00ff;
public const ushort BG_I2C_MONITOR_NACK = 0x0100;
public const uint BG_READ_I2C_NO_STOP = 0x00010000;
public static int bg_i2c_read (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    ushort[]   data_in
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_in_max_bytes = (int)tp_min(max_bytes, data_in.Length);
    return net_bg_i2c_read(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_in_max_bytes, data_in);
}

public static int bg_i2c_read_data_timing (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    ushort[]   data_in,
    int        max_timing,
    uint[]     data_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_in_max_bytes = (int)tp_min(max_bytes, data_in.Length);
    int data_timing_max_timing = (int)tp_min(max_timing, data_timing.Length);
    return net_bg_i2c_read_data_timing(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_in_max_bytes, data_in, data_timing_max_timing, data_timing);
}

public static int bg_i2c_read_bit_timing (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    ushort[]   data_in,
    int        max_timing,
    uint[]     bit_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_in_max_bytes = (int)tp_min(max_bytes, data_in.Length);
    int bit_timing_max_timing = (int)tp_min(max_timing, bit_timing.Length);
    return net_bg_i2c_read_bit_timing(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_in_max_bytes, data_in, bit_timing_max_timing, bit_timing);
}


/*=========================================================================
| SPI API
 ========================================================================*/
// enum BeagleSpiSSPolarity  (from declaration above)
//     BG_SPI_SS_ACTIVE_LOW  = 0
//     BG_SPI_SS_ACTIVE_HIGH = 1

// enum BeagleSpiSckSamplingEdge  (from declaration above)
//     BG_SPI_SCK_SAMPLING_EDGE_RISING  = 0
//     BG_SPI_SCK_SAMPLING_EDGE_FALLING = 1

// enum BeagleSpiBitorder  (from declaration above)
//     BG_SPI_BITORDER_MSB = 0
//     BG_SPI_BITORDER_LSB = 1

public static int bg_spi_configure (
    int                       beagle,
    BeagleSpiSSPolarity       ss_polarity,
    BeagleSpiSckSamplingEdge  sck_sampling_edge,
    BeagleSpiBitorder         bitorder
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_spi_configure(beagle, ss_polarity, sck_sampling_edge, bitorder);
}

public static int bg_spi_read (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        mosi_max_bytes,
    byte[]     data_mosi,
    int        miso_max_bytes,
    byte[]     data_miso
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_mosi_mosi_max_bytes = (int)tp_min(mosi_max_bytes, data_mosi.Length);
    int data_miso_miso_max_bytes = (int)tp_min(miso_max_bytes, data_miso.Length);
    return net_bg_spi_read(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_mosi_mosi_max_bytes, data_mosi, data_miso_miso_max_bytes, data_miso);
}

public static int bg_spi_read_data_timing (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        mosi_max_bytes,
    byte[]     data_mosi,
    int        miso_max_bytes,
    byte[]     data_miso,
    int        max_timing,
    uint[]     data_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_mosi_mosi_max_bytes = (int)tp_min(mosi_max_bytes, data_mosi.Length);
    int data_miso_miso_max_bytes = (int)tp_min(miso_max_bytes, data_miso.Length);
    int data_timing_max_timing = (int)tp_min(max_timing, data_timing.Length);
    return net_bg_spi_read_data_timing(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_mosi_mosi_max_bytes, data_mosi, data_miso_miso_max_bytes, data_miso, data_timing_max_timing, data_timing);
}

public static int bg_spi_read_bit_timing (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        mosi_max_bytes,
    byte[]     data_mosi,
    int        miso_max_bytes,
    byte[]     data_miso,
    int        max_timing,
    uint[]     bit_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int data_mosi_mosi_max_bytes = (int)tp_min(mosi_max_bytes, data_mosi.Length);
    int data_miso_miso_max_bytes = (int)tp_min(miso_max_bytes, data_miso.Length);
    int bit_timing_max_timing = (int)tp_min(max_timing, bit_timing.Length);
    return net_bg_spi_read_bit_timing(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, data_mosi_mosi_max_bytes, data_mosi, data_miso_miso_max_bytes, data_miso, bit_timing_max_timing, bit_timing);
}


/*=========================================================================
| USB API
 ========================================================================*/
/* USB packet PID definitions */
public const byte BG_USB_PID_OUT = 0xe1;
public const byte BG_USB_PID_IN = 0x69;
public const byte BG_USB_PID_SOF = 0xa5;
public const byte BG_USB_PID_SETUP = 0x2d;
public const byte BG_USB_PID_DATA0 = 0xc3;
public const byte BG_USB_PID_DATA1 = 0x4b;
public const byte BG_USB_PID_DATA2 = 0x87;
public const byte BG_USB_PID_MDATA = 0x0f;
public const byte BG_USB_PID_ACK = 0xd2;
public const byte BG_USB_PID_NAK = 0x5a;
public const byte BG_USB_PID_STALL = 0x1e;
public const byte BG_USB_PID_NYET = 0x96;
public const byte BG_USB_PID_PRE = 0x3c;
public const byte BG_USB_PID_ERR = 0x3c;
public const byte BG_USB_PID_SPLIT = 0x78;
public const byte BG_USB_PID_PING = 0xb4;
public const byte BG_USB_PID_EXT = 0xf0;
public const byte BG_USB_PID_CORRUPTED = 0xff;
/* The following codes are returned for USB 12 and 480 captures */
public const uint BG_READ_USB_ERR_BAD_SIGNALS = 0x00010000;
public const uint BG_READ_USB_ERR_BAD_PID = 0x00200000;
public const uint BG_READ_USB_ERR_BAD_CRC = 0x00400000;
/* The following codes are only returned for USB 12 captures */
public const uint BG_READ_USB_ERR_BAD_SYNC = 0x00020000;
public const uint BG_READ_USB_ERR_BIT_STUFF = 0x00040000;
public const uint BG_READ_USB_ERR_FALSE_EOP = 0x00080000;
public const uint BG_READ_USB_ERR_LONG_EOP = 0x00100000;
/* The following codes are only returned for USB 480 captures */
public const uint BG_READ_USB_TRUNCATION_MODE = 0x20000000;
public const uint BG_READ_USB_END_OF_CAPTURE = 0x40000000;
/* The following events are returned for USB 12 and 480 captures */
public const uint BG_EVENT_USB_HOST_DISCONNECT = 0x00000100;
public const uint BG_EVENT_USB_TARGET_DISCONNECT = 0x00000200;
public const uint BG_EVENT_USB_HOST_CONNECT = 0x00000400;
public const uint BG_EVENT_USB_TARGET_CONNECT = 0x00000800;
public const uint BG_EVENT_USB_RESET = 0x00001000;
/* USB 480 specific event codes */
public const uint BG_EVENT_USB_CHIRP_J = 0x00002000;
public const uint BG_EVENT_USB_CHIRP_K = 0x00004000;
public const uint BG_EVENT_USB_SPEED_UNKNOWN = 0x00008000;
public const uint BG_EVENT_USB_LOW_SPEED = 0x00010000;
public const uint BG_EVENT_USB_FULL_SPEED = 0x00020000;
public const uint BG_EVENT_USB_HIGH_SPEED = 0x00040000;
public const uint BG_EVENT_USB_LOW_OVER_FULL_SPEED = 0x00080000;
public const uint BG_EVENT_USB_SUSPEND = 0x00100000;
public const uint BG_EVENT_USB_RESUME = 0x00200000;
public const uint BG_EVENT_USB_KEEP_ALIVE = 0x00400000;
public const uint BG_EVENT_USB_DIGITAL_INPUT = 0x00800000;
public const uint BG_EVENT_USB_OTG_HNP = 0x01000000;
public const uint BG_EVENT_USB_OTG_SRP_DATA_PULSE = 0x02000000;
public const uint BG_EVENT_USB_OTG_SRP_VBUS_PULSE = 0x04000000;
public const uint BG_EVENT_USB_DIGITAL_INPUT_MASK = 0x0000000f;

/*=========================================================================
| USB 12 API
 ========================================================================*/
public static int bg_usb12_read (
    int        beagle,
    ref uint   status,
    ref uint   events,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    byte[]     packet
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int packet_max_bytes = (int)tp_min(max_bytes, packet.Length);
    return net_bg_usb12_read(beagle, ref status, ref events, ref time_sop, ref time_duration, ref time_dataoffset, packet_max_bytes, packet);
}

public static int bg_usb12_read_data_timing (
    int        beagle,
    ref uint   status,
    ref uint   events,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    byte[]     packet,
    int        max_timing,
    uint[]     data_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int packet_max_bytes = (int)tp_min(max_bytes, packet.Length);
    int data_timing_max_timing = (int)tp_min(max_timing, data_timing.Length);
    return net_bg_usb12_read_data_timing(beagle, ref status, ref events, ref time_sop, ref time_duration, ref time_dataoffset, packet_max_bytes, packet, data_timing_max_timing, data_timing);
}

public static int bg_usb12_read_bit_timing (
    int        beagle,
    ref uint   status,
    ref uint   events,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    byte[]     packet,
    int        max_timing,
    uint[]     bit_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int packet_max_bytes = (int)tp_min(max_bytes, packet.Length);
    int bit_timing_max_timing = (int)tp_min(max_timing, bit_timing.Length);
    return net_bg_usb12_read_bit_timing(beagle, ref status, ref events, ref time_sop, ref time_duration, ref time_dataoffset, packet_max_bytes, packet, bit_timing_max_timing, bit_timing);
}


/*=========================================================================
| USB 480 API
 ========================================================================*/
/* General constants */
public const byte BG_USB480_TRUNCATION_LENGTH = 4;
/* Capture modes */
// enum BeagleUsb480CaptureMode  (from declaration above)
//     BG_USB480_CAPTURE_REALTIME                 = 0
//     BG_USB480_CAPTURE_REALTIME_WITH_PROTECTION = 1
//     BG_USB480_CAPTURE_DELAYED_DOWNLOAD         = 2

/* Target speeds */
// enum BeagleUsb480TargetSpeed  (from declaration above)
//     BG_USB480_AUTO_SPEED_DETECT = 0
//     BG_USB480_LOW_SPEED         = 1
//     BG_USB480_FULL_SPEED        = 2
//     BG_USB480_HIGH_SPEED        = 3

public static int bg_usb480_capture_configure (
    int                      beagle,
    BeagleUsb480CaptureMode  capture_mode,
    BeagleUsb480TargetSpeed  target_speed
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_usb480_capture_configure(beagle, capture_mode, target_speed);
}

/* Digital output configuration */
public const byte BG_USB480_DIGITAL_OUT_ENABLE_PIN1 = 0x01;
public const byte BG_USB480_DIGITAL_OUT_PIN1_ACTIVE_HIGH = 0x01;
public const byte BG_USB480_DIGITAL_OUT_PIN1_ACTIVE_LOW = 0x00;
public const byte BG_USB480_DIGITAL_OUT_ENABLE_PIN2 = 0x02;
public const byte BG_USB480_DIGITAL_OUT_PIN2_ACTIVE_HIGH = 0x02;
public const byte BG_USB480_DIGITAL_OUT_PIN2_ACTIVE_LOW = 0x00;
public const byte BG_USB480_DIGITAL_OUT_ENABLE_PIN3 = 0x04;
public const byte BG_USB480_DIGITAL_OUT_PIN3_ACTIVE_HIGH = 0x04;
public const byte BG_USB480_DIGITAL_OUT_PIN3_ACTIVE_LOW = 0x00;
public const byte BG_USB480_DIGITAL_OUT_ENABLE_PIN4 = 0x08;
public const byte BG_USB480_DIGITAL_OUT_PIN4_ACTIVE_HIGH = 0x08;
public const byte BG_USB480_DIGITAL_OUT_PIN4_ACTIVE_LOW = 0x00;
public static int bg_usb480_digital_out_config (
    int   beagle,
    byte  out_enable_mask,
    byte  out_polarity_mask
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_usb480_digital_out_config(beagle, out_enable_mask, out_polarity_mask);
}

/* Digital output match pin configuration */
// enum BeagleUsb480DigitalOutMatchPins  (from declaration above)
//     BG_USB480_DIGITAL_OUT_MATCH_PIN3 = 3
//     BG_USB480_DIGITAL_OUT_MATCH_PIN4 = 4

/* Packet matching modes */
// enum BeagleUsb480MatchType  (from declaration above)
//     BG_USB480_MATCH_TYPE_DISABLED  = 0
//     BG_USB480_MATCH_TYPE_EQUAL     = 1
//     BG_USB480_MATCH_TYPE_NOT_EQUAL = 2

/* Digital ouput matching configuration */
[StructLayout(LayoutKind.Sequential)]
public struct BeagleUsb480PacketMatch {
    public BeagleUsb480MatchType pid_match_type;
    public byte                  pid_match_val;
    public BeagleUsb480MatchType dev_match_type;
    public byte                  dev_match_val;
    public BeagleUsb480MatchType ep_match_type;
    public byte                  ep_match_val;
}

/* Data match PID mask */
public const byte BG_USB480_DATA_MATCH_DATA0 = 0x01;
public const byte BG_USB480_DATA_MATCH_DATA1 = 0x02;
public const byte BG_USB480_DATA_MATCH_DATA2 = 0x04;
public const byte BG_USB480_DATA_MATCH_MDATA = 0x08;
[StructLayout(LayoutKind.Sequential)]
public struct BeagleUsb480DataMatch {
    public BeagleUsb480MatchType data_match_type;
    public byte                  data_match_pid;
    public byte[]                data;
    public byte[]                data_valid;
}

[StructLayout(LayoutKind.Sequential)]
private struct c_BeagleUsb480DataMatch {
    public BeagleUsb480MatchType   data_match_type;
    public byte                    data_match_pid;
    public ushort                  data_length;
    public IntPtr                  data;
    public ushort                  data_valid_length;
    public IntPtr                  data_valid;
}

private static c_BeagleUsb480DataMatch to_c_BeagleUsb480DataMatch (
    ref c_BeagleUsb480DataMatch c_s,
    ref BeagleUsb480DataMatch s,
    GCContext gcc,
    bool is_in_arg
)
{
    /* Process arrays */
    if (s.data != null) {
        c_s.data_length = (ushort)s.data.Length;
        GCHandle data_gch = GCHandle.Alloc(s.data, GCHandleType.Pinned);
        c_s.data = data_gch.AddrOfPinnedObject();
        gcc.add(data_gch);
    }
    else {
        c_s.data_length = 0;
        c_s.data = IntPtr.Zero;
    }
    if (s.data_valid != null) {
        c_s.data_valid_length = (ushort)s.data_valid.Length;
        GCHandle data_valid_gch = GCHandle.Alloc(s.data_valid, GCHandleType.Pinned);
        c_s.data_valid = data_valid_gch.AddrOfPinnedObject();
        gcc.add(data_valid_gch);
    }
    else {
        c_s.data_valid_length = 0;
        c_s.data_valid = IntPtr.Zero;
    }

    /* Process simple arguments */
    if (is_in_arg) {
        c_s.data_match_type = s.data_match_type;
        c_s.data_match_pid = s.data_match_pid;
    }

    return c_s;
}

public static int bg_usb480_digital_out_match (
    int                              beagle,
    BeagleUsb480DigitalOutMatchPins  pin_num,
    BeagleUsb480PacketMatch          packet_match,
    BeagleUsb480DataMatch            data_match
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    GCContext gcc = new GCContext();
    c_BeagleUsb480DataMatch c_data_match = new c_BeagleUsb480DataMatch();
    to_c_BeagleUsb480DataMatch(ref c_data_match, ref data_match, gcc, true);
    int ret = net_bg_usb480_digital_out_match(beagle, pin_num, ref packet_match, ref c_data_match);
    gcc.free();
    return ret;
}

/* Digital input pin configuration */
public const byte BG_USB480_DIGITAL_IN_ENABLE_PIN1 = 0x01;
public const byte BG_USB480_DIGITAL_IN_ENABLE_PIN2 = 0x02;
public const byte BG_USB480_DIGITAL_IN_ENABLE_PIN3 = 0x04;
public const byte BG_USB480_DIGITAL_IN_ENABLE_PIN4 = 0x08;
public static int bg_usb480_digital_in_config (
    int   beagle,
    byte  in_enable_mask
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_usb480_digital_in_config(beagle, in_enable_mask);
}

/* Hardware filtering configuration */
public const byte BG_USB480_HW_FILTER_PID_SOF = 0x01;
public const byte BG_USB480_HW_FILTER_PID_IN = 0x02;
public const byte BG_USB480_HW_FILTER_PID_PING = 0x04;
public const byte BG_USB480_HW_FILTER_PID_PRE = 0x08;
public const byte BG_USB480_HW_FILTER_PID_SPLIT = 0x10;
public const byte BG_USB480_HW_FILTER_SELF = 0x20;
public static int bg_usb480_hw_filter_config (
    int   beagle,
    byte  filter_enable_mask
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_usb480_hw_filter_config(beagle, filter_enable_mask);
}

public static int bg_usb480_hw_buffer_stats (
    int       beagle,
    ref uint  buffer_size,
    ref uint  buffer_usage,
    ref byte  buffer_full
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_usb480_hw_buffer_stats(beagle, ref buffer_size, ref buffer_usage, ref buffer_full);
}

public static int bg_usb480_read (
    int        beagle,
    ref uint   status,
    ref uint   events,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    int        max_bytes,
    byte[]     packet
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int packet_max_bytes = (int)tp_min(max_bytes, packet.Length);
    return net_bg_usb480_read(beagle, ref status, ref events, ref time_sop, ref time_duration, ref time_dataoffset, packet_max_bytes, packet);
}

public static int bg_usb480_reconstruct_timing (
    BeagleUsb480TargetSpeed  speed,
    int                      num_bytes,
    byte[]                   packet,
    int                      max_timing,
    uint[]                   bit_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int packet_num_bytes = (int)tp_min(num_bytes, packet.Length);
    int bit_timing_max_timing = (int)tp_min(max_timing, bit_timing.Length);
    return net_bg_usb480_reconstruct_timing(speed, packet_num_bytes, packet, bit_timing_max_timing, bit_timing);
}


/*=========================================================================
| MDIO API
 ========================================================================*/
// enum BeagleMdioClause  (from declaration above)
//     BG_MDIO_CLAUSE_22    = 0
//     BG_MDIO_CLAUSE_45    = 1
//     BG_MDIO_CLAUSE_ERROR = 2

public const byte BG_MDIO_OPCODE22_WRITE = 0x01;
public const byte BG_MDIO_OPCODE22_READ = 0x02;
public const byte BG_MDIO_OPCODE22_ERROR = 0xff;
public const byte BG_MDIO_OPCODE45_ADDR = 0x00;
public const byte BG_MDIO_OPCODE45_WRITE = 0x01;
public const byte BG_MDIO_OPCODE45_READ_POSTINC = 0x02;
public const byte BG_MDIO_OPCODE45_READ = 0x03;
/* Read the next MDIO frame. */
public static int bg_mdio_read (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    ref uint   data_in
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_mdio_read(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, ref data_in);
}

public static int bg_mdio_read_bit_timing (
    int        beagle,
    ref uint   status,
    ref ulong  time_sop,
    ref ulong  time_duration,
    ref uint   time_dataoffset,
    ref uint   data_in,
    int        max_timing,
    uint[]     bit_timing
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    int bit_timing_max_timing = (int)tp_min(max_timing, bit_timing.Length);
    return net_bg_mdio_read_bit_timing(beagle, ref status, ref time_sop, ref time_duration, ref time_dataoffset, ref data_in, bit_timing_max_timing, bit_timing);
}

/*
 * Parse the raw MDIO data into the standard format.
 * This function will fill the supplied fields as per
 * the constants defined above.  If the raw data contains
 * a malformed turnaround field, the caller will be
 * notified of the error through the return value of
 * this function (BG_MDIO_BAD_TURNAROUND).
 */
public static int bg_mdio_parse (
    uint        packet,
    ref byte    clause,
    ref byte    opcode,
    ref byte    addr1,
    ref byte    addr2,
    ref ushort  data
)
{
    if (!BG_LIBRARY_LOADED) return (int)BeagleStatus.BG_INCOMPATIBLE_LIBRARY;
    return net_bg_mdio_parse(packet, ref clause, ref opcode, ref addr1, ref addr2, ref data);
}


/*=========================================================================
| NATIVE DLL BINDINGS
 ========================================================================*/
[DllImport ("beagle")]
private static extern int net_bg_find_devices (int num_devices, [Out] ushort[] devices);

[DllImport ("beagle")]
private static extern int net_bg_find_devices_ext (int num_devices, [Out] ushort[] devices, int num_ids, [Out] uint[] unique_ids);

[DllImport ("beagle")]
private static extern int net_bg_open (int port_number);

[DllImport ("beagle")]
private static extern int net_bg_open_ext (int port_number, ref BeagleExt bg_ext);

[DllImport ("beagle")]
private static extern int net_bg_close (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_port (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_features (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_unique_id_to_features (uint unique_id);

[DllImport ("beagle")]
private static extern uint net_bg_unique_id (int beagle);

[DllImport ("beagle")]
private static extern IntPtr net_bg_status_string (int status);

[DllImport ("beagle")]
private static extern int net_bg_version (int beagle, ref BeagleVersion version);

[DllImport ("beagle")]
private static extern int net_bg_latency (int beagle, uint milliseconds);

[DllImport ("beagle")]
private static extern int net_bg_timeout (int beagle, uint milliseconds);

[DllImport ("beagle")]
private static extern uint net_bg_sleep_ms (uint milliseconds);

[DllImport ("beagle")]
private static extern int net_bg_target_power (int beagle, byte power_flag);

[DllImport ("beagle")]
private static extern int net_bg_host_ifce_speed (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_dev_addr (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_host_buffer_size (int beagle, uint num_bytes);

[DllImport ("beagle")]
private static extern int net_bg_host_buffer_free (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_host_buffer_used (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_commtest (int beagle, int num_samples, int delay_count);

[DllImport ("beagle")]
private static extern int net_bg_enable (int beagle, BeagleProtocol protocol);

[DllImport ("beagle")]
private static extern int net_bg_disable (int beagle);

[DllImport ("beagle")]
private static extern int net_bg_samplerate (int beagle, int samplerate_khz);

[DllImport ("beagle")]
private static extern int net_bg_bit_timing_size (BeagleProtocol protocol, int num_data_bytes);

[DllImport ("beagle")]
private static extern int net_bg_i2c_pullup (int beagle, byte pullup_flag);

[DllImport ("beagle")]
private static extern int net_bg_i2c_read (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] ushort[] data_in);

[DllImport ("beagle")]
private static extern int net_bg_i2c_read_data_timing (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] ushort[] data_in, int max_timing, [Out] uint[] data_timing);

[DllImport ("beagle")]
private static extern int net_bg_i2c_read_bit_timing (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] ushort[] data_in, int max_timing, [Out] uint[] bit_timing);

[DllImport ("beagle")]
private static extern int net_bg_spi_configure (int beagle, BeagleSpiSSPolarity ss_polarity, BeagleSpiSckSamplingEdge sck_sampling_edge, BeagleSpiBitorder bitorder);

[DllImport ("beagle")]
private static extern int net_bg_spi_read (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int mosi_max_bytes, [Out] byte[] data_mosi, int miso_max_bytes, [Out] byte[] data_miso);

[DllImport ("beagle")]
private static extern int net_bg_spi_read_data_timing (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int mosi_max_bytes, [Out] byte[] data_mosi, int miso_max_bytes, [Out] byte[] data_miso, int max_timing, [Out] uint[] data_timing);

[DllImport ("beagle")]
private static extern int net_bg_spi_read_bit_timing (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int mosi_max_bytes, [Out] byte[] data_mosi, int miso_max_bytes, [Out] byte[] data_miso, int max_timing, [Out] uint[] bit_timing);

[DllImport ("beagle")]
private static extern int net_bg_usb12_read (int beagle, ref uint status, ref uint events, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] byte[] packet);

[DllImport ("beagle")]
private static extern int net_bg_usb12_read_data_timing (int beagle, ref uint status, ref uint events, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] byte[] packet, int max_timing, [Out] uint[] data_timing);

[DllImport ("beagle")]
private static extern int net_bg_usb12_read_bit_timing (int beagle, ref uint status, ref uint events, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] byte[] packet, int max_timing, [Out] uint[] bit_timing);

[DllImport ("beagle")]
private static extern int net_bg_usb480_capture_configure (int beagle, BeagleUsb480CaptureMode capture_mode, BeagleUsb480TargetSpeed target_speed);

[DllImport ("beagle")]
private static extern int net_bg_usb480_digital_out_config (int beagle, byte out_enable_mask, byte out_polarity_mask);

[DllImport ("beagle")]
private static extern int net_bg_usb480_digital_out_match (int beagle, BeagleUsb480DigitalOutMatchPins pin_num, ref BeagleUsb480PacketMatch packet_match, ref c_BeagleUsb480DataMatch data_match);

[DllImport ("beagle")]
private static extern int net_bg_usb480_digital_in_config (int beagle, byte in_enable_mask);

[DllImport ("beagle")]
private static extern int net_bg_usb480_hw_filter_config (int beagle, byte filter_enable_mask);

[DllImport ("beagle")]
private static extern int net_bg_usb480_hw_buffer_stats (int beagle, ref uint buffer_size, ref uint buffer_usage, ref byte buffer_full);

[DllImport ("beagle")]
private static extern int net_bg_usb480_read (int beagle, ref uint status, ref uint events, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, int max_bytes, [Out] byte[] packet);

[DllImport ("beagle")]
private static extern int net_bg_usb480_reconstruct_timing (BeagleUsb480TargetSpeed speed, int num_bytes, [In] byte[] packet, int max_timing, [Out] uint[] bit_timing);

[DllImport ("beagle")]
private static extern int net_bg_mdio_read (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, ref uint data_in);

[DllImport ("beagle")]
private static extern int net_bg_mdio_read_bit_timing (int beagle, ref uint status, ref ulong time_sop, ref ulong time_duration, ref uint time_dataoffset, ref uint data_in, int max_timing, [Out] uint[] bit_timing);

[DllImport ("beagle")]
private static extern int net_bg_mdio_parse (uint packet, ref byte clause, ref byte opcode, ref byte addr1, ref byte addr2, ref ushort data);


} // class BeagleApi

} // namespace TotalPhase
