/*
 *==============================================================================
 * CERN (BE-CO-HT)
 * Source file for Multiboot
 *==============================================================================
 *
 * author: Theodor Stana (t.stana@cern.ch)
 *
 * date of creation: 2013-09-12
 *
 * version: 1.0
 *
 * description: 
 *
 * dependencies:
 *
 * references:
 * 
 *==============================================================================
 * GNU LESSER GENERAL PUBLIC LICENSE
 *==============================================================================
 * This source file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version. This source is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details. You should have
 * received a copy of the GNU Lesser General Public License along with this
 * source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
 *==============================================================================
 * last changes:
 *    2013-09-12   Theodor Stana     t.stana@cern.ch     File created
 *==============================================================================
 * TODO: - 
 *==============================================================================
 */
 
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <sys/time.h>
#include <math.h>

#include "vbcp.h"

static int vfd;

static uint64_t get_ticks()
{
     struct timeval tv;
     gettimeofday( &tv, NULL );

     return (uint64_t) tv.tv_sec * 1000000ULL + (uint64_t) tv.tv_usec;
}


uint8_t spi_transfer(uint32_t cs, uint32_t dat)
{
  int retval = 0;
  vbcp_write(vfd, 4, 0x90, ((cs << 10) | 0x100) | dat);
  while ( !(retval & (1<<9)) )
  {
    retval = vbcp_read(vfd, 4, 0x90);
  }
  return retval & 0xFF;
}

void flash_write(int nrbytes, uint32_t bytes, uint8_t *dat)
{
  int i;

  spi_transfer(0,0);
  spi_transfer(1,0x06);
  spi_transfer(0,0);
  spi_transfer(1,0x02);
  spi_transfer(1,(bytes & 0xFF0000) >> 16);
  spi_transfer(1,(bytes & 0xFF00) >> 8);
  spi_transfer(1,(bytes & 0xFF));
  for ( i = 0; i < nrbytes; i++ )
  {
    spi_transfer(1,dat[i]);
  }
  spi_transfer(0,0);
}

void flash_read(int nrbytes, uint32_t bytes, uint8_t *dat)
{
  int cnt = 0;
  int i;
  spi_transfer(0,0);
  spi_transfer(1,0x0b);
  spi_transfer(1,(bytes & 0xFF0000) >> 16);
  spi_transfer(1,(bytes & 0xFF00) >> 8);
  spi_transfer(1,(bytes & 0xFF));
  spi_transfer(1,0);
  for ( i = 0; i < nrbytes; i++ )
  {
    dat[i] = spi_transfer(1, 0);
  }
  spi_transfer(0,0);
}

void flash_serase(uint32_t bytes)
{
  spi_transfer(0,0);
  spi_transfer(1,0x06);
  spi_transfer(0,0);
  spi_transfer(1,0xD8);
  spi_transfer(1,(bytes & 0xFF0000) >> 16);
  spi_transfer(1,(bytes & 0xFF00) >> 8);
  spi_transfer(1,(bytes & 0xFF));
  spi_transfer(0,0);
}

uint8_t flash_rsr()
{
  uint8_t retval;
  spi_transfer(0,0);
  spi_transfer(1,0x05);
  retval = spi_transfer(1,0);
  spi_transfer(0,0);
  return retval;
}

int main()
{
  uint64_t ts, tf, t0, t1;
  double te[32], tw[6000];
  double temean = 0,
          twmean = 0;
  int i = 0;
  FILE *f;
  char fdat[520];
  uint8_t dat[256];
  uint32_t x=0;
  uint32_t pages = 0;
  uint32_t erases = 0;
  uint32_t addr = 0;
  memset(dat,0,sizeof(dat));

  ts = get_ticks();

  vfd = vbcp_open();
  if (vfd == -1) {
    printf("VBCP not open!\n");
    return -1;
  }


  t0 = get_ticks();
  flash_read(256, addr, dat);
  t1 = get_ticks();

  for (i = 0; i < 256; i++)
  {
    printf("0x%02x ", dat[i]);
  }
  printf("\n\n");
  printf("\n");
//  printf("t0: %lld\n", t0);
//  printf("t1: %lld\n", t1);
  printf("read time : %2.6f s\n", (double)(t1 - t0) / pow(10,6));

  f = fopen("conv.txt", "r");
  t0 = get_ticks();
  while (pages<256)
  {
//  while (!feof(f))
//  {
    printf("addr=%x\n", addr);
    fgets(fdat, 520, f);
//    printf("%s\n", fdat);
    for (i=0;i<256;i++)
    {
      if (strlen(fdat) < 512 && i==strlen(fdat)/2)
        break;
      sscanf(&fdat[2*i], "%2hhx", &dat[i]);
//      printf("%02x",dat[i]);
//      printf("%d\n",strlen(fdat));
//      printf("%d\n", bytes);

    }

    if ( addr % 0x10000 == 0 )
    {
      printf("erase\n");
      t0 = get_ticks();
      flash_serase(addr);
      while ( flash_rsr() & 0x01 )
        ;
      t1 = get_ticks();
//      printf("t0: %lld\n", t0);
//      printf("t1: %lld\n", t1);
      printf("erase time: %2.6f s\n", (double)(t1 - t0) / pow(10,6));
      te[erases] = (double)(t1 - t0) / pow(10,6);
      erases++;
    }

    t0 = get_ticks();
    flash_write(i, addr, dat);
    while ( flash_rsr() & 0x01 )
      ;
    t1 = get_ticks();
//    printf("t0: %lld\n", t0);
//    printf("t1: %lld\n", t1);
    printf("write time: %2.6f s\n", (double)(t1 - t0) / pow(10,6));

    tw[pages] = (double)(t1 - t0) / pow(10,6);
    pages++;
    addr += 256;

//    printf("\n");
//    printf("%d\n", bytes);
  }
  t1 = get_ticks();
  fclose(f);
  printf("erases: %d\n", erases);
  printf("pages: %d\n", pages);
  for (i = 0; i < erases; i++)
  {
    temean += te[i];
  }
  temean = temean / erases;
  for (i = 0; i < pages; i++)
  {
    twmean += tw[i];
  }
  twmean = twmean / pages;
  printf("mean erase time: %2.6f s\n", temean);
  printf("mean write time: %2.6f s\n", twmean);
  tf = get_ticks();
  printf("time: %2.6f s\n", (double)(tf - ts) / pow(10,6));

  vbcp_close(vfd);

  return 0;
}

