/*
 *==============================================================================
 * CERN (BE-CO-HT)
 * Source file for VME Board Control Protocol
 *==============================================================================
 *
 * author: Theodor Stana (t.stana@cern.ch)
 *
 * date of creation: 2013-09-11
 *
 * version: 1.0
 *
 * description: 
 *
 * dependencies:
 *
 * references:
 * 
 *==============================================================================
 * GNU LESSER GENERAL PUBLIC LICENSE
 *==============================================================================
 * This source file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version. This source is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details. You should have
 * received a copy of the GNU Lesser General Public License along with this
 * source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
 *==============================================================================
 * last changes:
 *    2013-09-11   Theodor Stana     t.stana@cern.ch     File created
 *==============================================================================
 * TODO: - 
 *==============================================================================
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>

#define handle_error(msg) \
    do { perror(msg); return -1; } while (0)

#define BUFSIZE 64
#define INPSIZE 16





int
vbcp_open()
{
  int sfd = -1;
  char buf[BUFSIZE];
  struct addrinfo hints;
  struct addrinfo *res;
  char input[INPSIZE];

  memset(&hints, 0, sizeof(struct addrinfo));
  hints.ai_flags = AI_PASSIVE;
  hints.ai_family = AF_INET;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_protocol = 0;
  hints.ai_addr = NULL;
  hints.ai_canonname = NULL;
  hints.ai_next = NULL;

  if ( getaddrinfo("elma", "telnet", &hints, &res) != 0 )
    handle_error("getaddrinfo");

  sfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
  if (sfd == -1 )
    handle_error("init");

  if ( connect(sfd, res->ai_addr, res->ai_addrlen) == -1 )
    handle_error("connect");

  memset(buf, 0, BUFSIZE);
  if ( recv(sfd, buf, BUFSIZE, 0) == -1 )
    handle_error("recv");

//  printf("%s\n", buf);

  if ( strstr(buf, "login") == NULL )
  {
    printf("login not found\n");
    return -1;
  }

  while (1)
  {
    while (1)
      {
        memset(input, 0, INPSIZE);
        printf("login: ");
        fgets(input, INPSIZE, stdin);
        strcpy(input + (strlen(input)-1), "\r\n");

        if ( send(sfd, input, 7, 0) == -1 )
          handle_error("account send");

        memset(buf, 0, BUFSIZE);
        if ( recv(sfd, buf, BUFSIZE, 0) == -1 )
          handle_error("password");
      //  printf("%s\n",buf);

        if ( strstr(buf, "Wrong username") != NULL )
        {
          printf("Wrong username\n");
        }
        else if ( strstr(buf, "password") != NULL )
        {
          break;
        }
  //      memset(buf, 0, BUFSIZE);
  //      if ( recv(sfd, buf, BUFSIZE, 0) == -1 )
  //        handle_error("recv");
  //      if ( strstr(buf, "login") == NULL )
  //      {
  //        printf("%s", buf);
  //      }
  //      else
  //      {
  //        break;
  //      }
      }

    memset(input, 0, INPSIZE);
    printf("password: ");
    fgets(input, INPSIZE, stdin);
    strcpy(input + (strlen(input)-1), "\r\n");

    if ( send(sfd, input, 7, 0) == -1 )
      handle_error("passwd send");

    memset(buf, 0, BUFSIZE);
    if ( recv(sfd, buf, BUFSIZE, 0) == -1 )
      handle_error("cmdline");

    if ( strstr(buf, "Wrong password") != NULL )
    {
      printf("Wrong password\n");
    }
    else if ( strstr(buf, "%>") != NULL )
    {
      break;
//      printf("unexpected response: %s\n", buf);
//      return -1;
    }
  }

  return sfd;
}




int
vbcp_write(int vfd, uint32_t slot, uint32_t addr, uint32_t val)
{
  char cmd[32];
  char resp[32];

  sprintf(cmd, "writereg %d %d %x\r\n", slot, (1+addr/4), val);
  //printf("%s", cmd);
  if ( send(vfd, cmd, strlen(cmd), 0) == -1 )
    handle_error("write error");

  if ( recv(vfd, resp, sizeof(resp), 0) == -1 )
    handle_error("write resp error");

  if ( strstr(resp, "Done!") == NULL )
    printf("not a good reply: %s\n", resp);
    return -1;

  return 0;
}




int
vbcp_read(int vfd, uint32_t slot, uint32_t addr)
{
  char cmd[32];
  char resp[32];
  char hex[16];
  uint32_t retval;

  sprintf(cmd, "readreg %d %d\r\n", slot, (1+addr/4));
  //printf("cmd: %s", cmd);
//  for (retval = 0; cmd[retval] != '\0'; retval++)
//  {
//    printf("%02x", cmd[retval]);
//  }
//  printf("\n");
  if ( send(vfd, cmd, strlen(cmd), 0) == -1 )
    handle_error("read error");

  if ( recv(vfd, resp, sizeof(resp), 0) == -1 )
    handle_error("read resp error");

  if ( strstr(resp, "Read Data:") == NULL )
  {
    printf("not a good reply: %s\n", resp);
    return -1;
  }
  else
  {
    //printf("resp: %s\n", resp);
    strncpy(hex, resp+14, 8);
    //printf("hex: %s\n", hex);
    hex[8] = '\0';
    //printf("stripped: %s\n", hex);
    sscanf(hex, "%X", &retval);
    //printf("%x\n", retval);
  }

  return retval;
}





void
vbcp_close(int vfd)
{
  close(vfd);
}




//int main()
//{
//  int vfd;
//  int reg;
//
//  vfd = vbcp_open();
//  vbcp_write(vfd, 4, 0x8c, 0xaabbccdd);
//  reg = vbcp_read(vfd, 4, 0x8c);
//  printf("%x\n", reg);
//  vbcp_close(vfd);
//
//  return 0;
//}
